package trivial;

import static common.Log.LOGGER_NAME_TEST;
import static common.Log.LOG_ON;
import static common.Log.TEST;
import static org.junit.Assert.assertTrue;

import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.nio.channels.SocketChannel;

import org.apache.log4j.Level;
import org.junit.Ignore;
import org.junit.Test;

import common.FullDuplexMsgWorker;
import common.Log;
import common.ReadMessageStatus;
import json.TrivialQuestion;

public class TestReceiveOrSendAnswer {
	static public final int PORT = 5010;
	static public final int PORT2 = 5020;
	static public final String HOST = "localhost";
	static public final String NICK1 = "Pseudo1";
	static public final int ANSWERNUM1 = 1;
	static public final String NICK2 = "Pseudo2";
	static public final int ANSWERNUM2 = 2;


	private static boolean compareAnswer(AnswersList al1, int pos1 , AnswersList al2, int pos2 )  {
		if (al1.getAnswerNum(pos1) != al2.getAnswerNum(pos2)) {
			return false;
		}
		if (!al1.getNickname(pos1).equals(al2.getNickname(pos2))) {
			return false;
		}
		return true;

	}


	@Ignore
	@Test
	public void testReceiveAnswerFromPlayer() throws IOException, InterruptedException {
		testReceiveOrSend(TestId.RECEIVEANSWERFROMLPLAYER);
	}

	@Ignore
	@Test
	public void testSendCorrectAnswer() throws IOException, InterruptedException {
		testReceiveOrSend(TestId.SENDCORRECTANSWER);
	}


	private void testReceiveOrSend(TestId testId) throws IOException, InterruptedException {
		int deltaPort = 0;
		if (testId == TestId.SENDCORRECTANSWER) {
			deltaPort = 100;
		}
		Log.configureALogger(LOGGER_NAME_TEST, Level.TRACE);

		if (LOG_ON && TEST.isInfoEnabled()) {
			TEST.info("starting " + testId);
		}


		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": creating server");
		}
		// creating new server
		TrivialServer trivialServer = new TrivialServer(PORT+deltaPort, AppliServer.TRIVIALHTTPURL, testId);


		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": creating ServerSocketChannel");
		}
		// server: open listen channel
		trivialServer.playTest(TestId.OPENLISTENCHANNEL);

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": connect to server");
		}
		// connect to the server
		SocketChannel  rwChan = SocketChannel.open(new InetSocketAddress(InetAddress.getByName(HOST),PORT+deltaPort ));

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": accept");
		}
		// server accept new player
		trivialServer.playTest(TestId.ACCEPTNEWPLAYER);

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": sending new question");
		}
		// server: send a new question
		trivialServer.playTest(TestId.SENDNEWQUESTION);

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": receiving new question");
		}
		// receiving this question
		FullDuplexMsgWorker worker = new FullDuplexMsgWorker(rwChan);
		ReadMessageStatus status = ReadMessageStatus.ReadUnstarted;


		while ((status != ReadMessageStatus.ChannelClosed) && (status != ReadMessageStatus.ReadDataCompleted)) { 
			status = worker.readMessage();
		}

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": checking data");
		}
		assertTrue("Testting data completed" , status == ReadMessageStatus.ReadDataCompleted);

		assertTrue("is instance of Trivial Question", worker.getData() instanceof TrivialQuestion);


		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": sending answer 1");
		}
		// sending an answer
		AnswersList answersList = new AnswersList(NICK1, ANSWERNUM1);
		worker.sendMsg(MessType.ANSWERSLISTTYPE, answersList);

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": receiving answer 1");
		}
		// server: receive answer from player
		trivialServer.playTest(TestId.RECEIVEANSWERFROMLPLAYER);

		if (testId == TestId.SENDCORRECTANSWER) {
			if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
				TEST.trace("testing " + testId + ": sending answersList");
			}
			trivialServer.playTest(TestId.SENDCORRECTANSWER);
		}
		else {
			// server: send back the received data
			if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
				TEST.trace("testing " + testId + ": sending back answersList");
			}
			trivialServer.playTest(TestId.RECEIVEANSWERFROMLPLAYER2);
		}

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": receiving answersList");
		}
		// receiving the answer list of  the server	
		status = ReadMessageStatus.ReadUnstarted;


		while ((status != ReadMessageStatus.ChannelClosed) && (status != ReadMessageStatus.ReadDataCompleted)) { 
			status = worker.readMessage();
		}

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": checking data");
		}

		assertTrue("Testting data completed" , status == ReadMessageStatus.ReadDataCompleted);

		AnswersList serverAnswerList = (AnswersList) worker.getData();

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": answersList should contain 1 element");
		}

		assertTrue("Testting answer list contains 1 element", serverAnswerList.getNbAnswers()== 1);

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": answersList should contain answer 1");
		}
		assertTrue("testing local answer and server answer are the same", compareAnswer(answersList, 0, serverAnswerList, 0));




		// **************** Test with 2 clients
		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": Testing with 2 players");
		}

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": creating server");
		}
		// creating new server
		trivialServer = new TrivialServer(PORT2+deltaPort, AppliServer.TRIVIALHTTPURL, testId);

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": creating ServerSocketChannel");
		}
		// server: open listen channel
		trivialServer.playTest(TestId.OPENLISTENCHANNEL);


		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": connect to server");
		}
		// connect to the server
		rwChan = SocketChannel.open(new InetSocketAddress(InetAddress.getByName(HOST),PORT2+deltaPort ));

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": accept");
		}
		// server accept new player
		trivialServer.playTest(TestId.ACCEPTNEWPLAYER);

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": sending new question");
		}
		// server: send a new question
		trivialServer.playTest(TestId.SENDNEWQUESTION);

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": receiving new question");
		}
		// receiving this question
		worker = new FullDuplexMsgWorker(rwChan);
		status = ReadMessageStatus.ReadUnstarted;


		while ((status != ReadMessageStatus.ChannelClosed) && (status != ReadMessageStatus.ReadDataCompleted)) { 
			status = worker.readMessage();
		}

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": checking data");
		}
		assertTrue("Testting data completed" , status == ReadMessageStatus.ReadDataCompleted);
		assertTrue("is instance of Trivial Question", worker.getData() instanceof TrivialQuestion);


		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": sending answer 1");
		}
		// sending  answer 1
		AnswersList answersList1 = new AnswersList(NICK1, ANSWERNUM1);
		worker.sendMsg(MessType.ANSWERSLISTTYPE, answersList1);

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": connect 2nd player to server");
		}
		// 2nd connexion
		SocketChannel  rwChan2 = SocketChannel.open(new InetSocketAddress(InetAddress.getByName(HOST),PORT2+deltaPort ));
		FullDuplexMsgWorker worker2 = new FullDuplexMsgWorker(rwChan2);

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": accept 2nd player");
		}
		// server: accept this 2nd connexion
		trivialServer.playTest(TestId.ACCEPTNEWPLAYER);

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": sending answer 2");
		}
		// sending  answer 2
		AnswersList answersList2 = new AnswersList(NICK2, ANSWERNUM2);
		worker2.sendMsg(MessType.ANSWERSLISTTYPE, answersList2);

		Thread.sleep(2000); // after 2 seconds answer 2 should be revceived 

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": receiving answer 1 and answer 2");
		}
		// server: receive the 2 answers
		trivialServer.playTest(TestId.RECEIVEANSWERFROMLPLAYER);


		if (testId == TestId.SENDCORRECTANSWER) {
			if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
				TEST.trace("testing " + testId + ": sending answersList");
			}
			trivialServer.playTest(TestId.SENDCORRECTANSWER);
		}
		else {
			if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
				TEST.trace("testing " + testId + ": sending back the 2 answers");
			}
			// server: send back the received data
			trivialServer.playTest(TestId.RECEIVEANSWERFROMLPLAYER2);
		}


		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": receiving answersList");
		}
		// client 1 receiving the answer list of  the server	
		status = ReadMessageStatus.ReadUnstarted;


		while ((status != ReadMessageStatus.ChannelClosed) && (status != ReadMessageStatus.ReadDataCompleted)) { 
			status = worker.readMessage();
		}

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": checking data");
		}

		assertTrue("Testting data completed" , status == ReadMessageStatus.ReadDataCompleted);

		AnswersList serverAnswersList1 = (AnswersList) worker.getData();


		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": answersList should contain 2 elements");
		}
		assertTrue("Testting server answers list contains 2 elements", serverAnswersList1.getNbAnswers()== 2);


		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing " + testId + ": answersList should contain answer 1 and answer 2");
		}

		// answersList1 may be on 1st or 2nd position of  serverAnswerList1
		assertTrue("testing server answers list contains the answersList of player 1 and player 2 ", 
				(compareAnswer(answersList1, 0, serverAnswersList1, 0) && compareAnswer(answersList2, 0, serverAnswersList1, 1)) ||
				(compareAnswer(answersList1, 0, serverAnswersList1, 1) && compareAnswer(answersList2, 0, serverAnswersList1, 0)));


		if (LOG_ON && TEST.isInfoEnabled()) {
			TEST.info("End of " + testId);
		}

	}



}
